
function [] = load_sim_data()
    global("ui_controls");
    global("N_RUNS");
    global("model_input");
    global("model_output");
    global("in_variables_statistics");
    global("out_variables");
    global("model_fit_fkt");             

    filename = uigetfile(["*.msci"], model_dir);
    if (filename=="") return; end
    try
        load(filename, "N_RUNS", "model_input", "model_output", "in_variables_statistics", "out_variables", "sim_time", "model_executable", "montesci_version_vv", "model_version_s", "model_fit_fkt");
        try
            xx = ui_controls(ID_WINDOW_LIST)(ID_PLOT_WINDOW);
        catch
            ui_controls(ID_B_PLOT_MAIN).enable= "on";
        end      

        try
            xx = ui_controls(ID_WINDOW_LIST)(ID_VAR_EXPLORER)
        catch
            ui_controls(ID_B_SAVE).enable= "on";
        end      
    end

    if ( (N_RUNS<2) | (length(N_RUNS)==0) )
        error_log_text("102: Awquard data in file - is this a MonteSci data file?");
        ui_controls(ID_B_PLOT_MAIN).enable= "off";
        return;
    end;    


    if (length(filename) > 50)
        filename = "..." + part(filename, length(filename)-50 : 1: length(filename));
    end

    log_text("Successfully loaded """+filename+""".",1,1)


    dd_str = sprintf("&nbsp &nbsp Simulation performed on: %d/%d/%d at %d:%d:%d", sim_time(1), sim_time(2), sim_time(3), sim_time(4), sim_time(5), sim_time(6) );
    log_text(dd_str,1,1)

    log_text(sprintf("&nbsp &nbsp Number of runs: %d", N_RUNS), 1,1);
    log_text(sprintf("&nbsp &nbsp Model: %s, version %s", shrink_string(model_executable, 50), model_version_s), 1,1);
    log_text(sprintf("&nbsp &nbsp Simulation performed with MonteSci: %s", montesci_version_vv), 1,1);
    log_text(" ", 1,1);
    log_text("<B>MonteSci is ready to plot.</B>", 1,1);    
    log_text(" ", 1,1);

    global("n_run");
    n_run = N_RUNS;

endfunction


function [] = save_sim_data()
    global("N_RUNS");
    global("model_input");
    global("model_output");
    global("in_variables_statistics");
    global("out_variables");
    global("model_version");
    global("model_fit_fkt");
    global("model_exec")
    sim_time = clock();

    model_executable = model_exec;
    montesci_version_vv = montesci_version;
    model_version_s = model_version;
    filename = uigetfile(["*.msci"], model_dir);
    if (filename=="") return; end

    if (length(strindex(filename, ".msci"))==0)  // append ".msci" if not already part of the name
        filename = filename + ".msci";
    end

    try
        save(filename, N_RUNS, model_input, model_output, in_variables_statistics, out_variables, sim_time, model_executable, montesci_version_vv, model_version_s, model_fit_fkt);
    catch
        error_log_text("103: File could not be saved.");
    end

    if (length(filename) > 50)
        filename = "..." + part(filename, length(filename)-50 : 1: length(filename));
    end
    log_text("Successfully saved: """+filename+""".",1,1)
endfunction



