function [] = perform_monte_carlo_sim()
  global('n_run');  
  global('ui_controls');
  global('N_RUNS');
  global('model_exec');
  global('msci_target_found'); // set
  global('msci_stop_simulation');
  msci_target_found = 0;
  msci_stop_simulation = 0;
  
  n_run = 0;
    
  if (model_dir == "")
    error_log_text("210: No model directory loaded. Use File/Create or File/Open to set the model directory.");
    ui_controls(ID_B_RUN).enable= "on";  
    return;
  end
 
  log_text("Running model init ""model.conf"" ...", 1, 1);

  err = exec(model_dir+"model.conf",'errcatch');
  if (err ~=0)          
    error_log_text("201: file '"model.conf'" could not be executed correctly. ");
    log_text ("Look for: "+shrink_string(model_dir+"model.conf", 70),1,1);
    return;
  end
  
  if (N_RUNS < 1)
    error_log_text("202: Number of Runs '"N_RUNS'" must be at least 1.");
    abort_execution(); 
  end  
  
  if (N_RUNS > N_SIM_MAX)
    error_log_text(   sprintf("203: For Stability reasons '"N_RUNS'" must be smaller than %d.", N_SIM_MAX));
    abort_execution(); 
  end 
  
  if (model_type == "scalar")  
    ui_controls(ID_B_RUN).String= "Stop Simulation";  
    ui_controls(ID_B_RUN).Callback = 'msci_stop_pressed()';
  end
   
  ui_controls(ID_B_LOAD).enable= "off";
  ui_controls(ID_B_SAVE).enable= "off";
  ui_controls(ID_B_PLOT_MAIN).enable= "off";
     
  if (prepare_variables(N_RUNS)) ui_controls(ID_B_RUN).enable= "on"; return; end; 
  break;
  
  log_text("Running model init """+model_pre_exec_file+"""...", 1, 1);
  try
    exec(model_pre_exec);
  catch
    error_log_text("204: File """+model_pre_exec_file+""" could not be executed correctly. ");
    [str, n, line, func] = lasterror(%t);
    if (func == "") func = model_exec_file; end;        
    log_text( sprintf("&nbsp &nbsp Function: %s at line %d", func, line), 1,1 );
    log_text("&nbsp &nbsp Error: "+str, 1,1);
    log_text("");
    ui_controls(ID_B_RUN).enable= "on";
    ui_controls(ID_B_LOAD).enable= "on";
    ui_controls(ID_B_RUN).String= "Run Simulation";  
    ui_controls(ID_B_RUN).Callback = 'perform_monte_carlo_sim()';
    return;
  end
  
  montesci_logfile_info("Starting Monte Carlo simulation...");
  log_text("Starting Monte Carlo simulation with model file """+ model_exec_file + """...", 1, 1);
     
  if (model_type == "scalar")
      
    sum_time = 0;
    try
      timer();
    catch 
    end
  
    for n_run = 1:1:N_RUNS
      
      if (call_model()~=0)
        model_error_handler();
        if (resume_on_error=="off") 
          ui_controls(ID_B_RUN).enable= "on";
          return; 
        end;
      end
      
      try
        sum_time = sum_time +  timer();
        timer();
      catch 
      end
      
      if (stop_criterion == "target")
        if (check_target()) msci_target_found =1; end;
      end
      
      if (msci_target_found == 1)
        target_found();
        return;
      end 
      
      if (msci_stop_simulation == 1)
        stop_execution();
        return;
      end
      
      if (modulo (n_run, 1+floor(N_RUNS/500)) ==0)
        progress_bar_per(n_run/N_RUNS*100);
      end         
      if (modulo (n_run, 1+floor(N_RUNS/15)) ==0)
        average_time = sum_time / n_run;
        rem_time = (N_RUNS - n_run)*average_time;   
        rem_h = floor(rem_time/3600);
        rem_min = floor(rem_time/60 - rem_h*60);
        rem_sec =  floor(rem_time - rem_h*3600 - rem_min*60);
        ttstr = sprintf("%d h, %d min, %d sec remaining", rem_h, rem_min, rem_sec);     
        log_text(sprintf("%d of %d, %.2f percent", n_run, N_RUNS, n_run/N_RUNS*100) + " done - "+ttstr, 1, 0); 
      end    
    end
    else
      if (model_type == "vector")
        log_text("<B>This is a vector model, there will be no progress calculation...</B>", 1, 1);
        try
          call_model();
          n_run = N_RUNS;                    
        catch
          model_error_handler();
          ui_controls(ID_B_RUN).enable= "on";
          return;
        end
      else
        error_log_text("205: Invalid model type (vector/scalar) is specified in simulation.conf.");
        ui_controls(ID_B_RUN).enable= "on";
        return;
      end
    end 
  
  log_text("Monte Carlo runs finished.", 1, 1);
  montesci_logfile_info("Monte Carlo simulation done.");
  progress_bar_per(100);
      
  log_text("Cleaning up after runs and preparing outputs...", 1, 1);  
  post_all_runs(N_RUNS);
  log_text("", 1, 1);
  log_text("<B>Monte Carlo simulation done, ready to plot.</B>", 1, 1);
  log_text("", 1, 1);
  
  ui_controls(ID_B_RUN).enable= "on";
  try
    xx = ui_controls(ID_WINDOW_LIST)(ID_PLOT_WINDOW);
    fill_variable_lists();    
  catch
    ui_controls(ID_B_PLOT_MAIN).enable= "on";
  end      
  
  try
    xx = ui_controls(ID_WINDOW_LIST)(ID_VAR_EXPLORER);
    fill_values(-1);    
  catch    
    ui_controls(ID_B_SAVE).enable= "on";
  end
      
  //ui_controls(ID_B_SAVE).enable= "on";
  ui_controls(ID_B_LOAD).enable= "on";
  ui_controls(ID_B_RUN).String= "Run Simulation";  
  ui_controls(ID_B_RUN).Callback = 'perform_monte_carlo_sim()';

endfunction 

function [] = msci_stop_pressed()
  global("ui_controls");
  global("msci_stop_simulation"); 
  msci_stop_simulation = 1;
  ui_controls(ID_B_RUN).String= "Run Simulation";  
  ui_controls(ID_B_RUN).Callback = 'perform_monte_carlo_sim()';
   
endfunction 

function [limits_met] = check_target()
  global('msci_target_found');
  global('n_run');
  global('out_variables');
  global('model_output');
  
  limits_met = %T;
       
  for i=1:1:out_variables(ID_OUTVAR_COUNT)  // check all variables
    limits = out_variables(ID_OUTVAR_LIMITS)(i);
    upper_limit = limits(2);
    lower_limit = limits(1);
    if (upper_limit > lower_limit)
      value = model_output(i)(n_run)
      if (value > upper_limit) | (value < lower_limit) limits_met = %F; end;
    end  
  end
  
   
endfunction

function [] = target_found()
  global('model_input');
  global('model_output');
  global('out_variables');
  global('n_run');
  global('N_RUNS');
  global('ui_controls');
  
  N_RUNS = n_run;
  log_text("<B>Simulation target found at run "+sprintf("%d", N_RUNS) + ", simulation halted.</B>",1,1);
      
  for i=1:1:out_variables(ID_OUTVAR_COUNT)  // save valid output variables       
    if (out_variables(ID_OUTVAR_TYPE)(i) == "scalar")
       model_output(i) = model_output(i)(1:N_RUNS);            
    end
    if (out_variables(ID_OUTVAR_TYPE) == "vector")
            
    end                      
  end
      
  for i=1:1:in_variables_statistics(ID_INVAR_COUNT)  // save valid input variables       
    model_input(i) = model_input(i)(1:N_RUNS);            
  end
      
  try
    xx = ui_controls(ID_WINDOW_LIST)(ID_PLOT_WINDOW);
    fill_variable_lists();
  catch
    ui_controls(ID_B_PLOT_MAIN).enable= "on";
  end      
  
  montesci_logfile_info("Monte Carlo simulation done.");
  progress_bar_per(100);
      
  log_text("Cleaning up after runs and preparing outputs...", 1, 1);  
  post_all_runs(N_RUNS);
  log_text("", 1, 1);
  log_text("<B>Monte Carlo simulation done, ready to plot.</B>", 1, 1);
  log_text("", 1, 1);
  
  ui_controls(ID_B_RUN).enable= "on";
  ui_controls(ID_B_SAVE).enable= "on";
  ui_controls(ID_B_LOAD).enable= "on";
  ui_controls(ID_B_RUN).String= "Run Simulation";  
  ui_controls(ID_B_RUN).Callback = 'perform_monte_carlo_sim()';
  
  progress_bar_per(100);
  
  try
    xx = ui_controls(ID_WINDOW_LIST)(ID_VAR_EXPLORER);
    fill_values(-1);    
  catch    
  end
   
endfunction


function [] = model_error_handler();
  global('model_input');
  global('model_output');
  global('out_variables');
  global('n_run');
  global('N_RUNS');
  global('ui_controls');
  
  if (model_type == "scalar")
    if (resume_on_error=="on")
      if (n_run == 1)
        log_text("Run 1: Model returned an error - outputs will set to <B> zero </B>. Resuming...", 1, 1);
      else
        log_text("<B>"+sprintf("Run %d: ", n_run)+"Model returned an error - outputs set to previous values</B>.", 1, 1);
      end 
      
      for i=1:1:out_variables(ID_OUTVAR_COUNT)
        global(out_variables(ID_OUTVAR_NAME)(i));
      end

      for i=1:1:out_variables(ID_OUTVAR_COUNT)  // assign result to output_vector
        if (n_run>1)                        
          model_output(i)(n_run) = model_output(i)(n_run-1);
        else
          model_output(i)(n_run) = 0;
        end        
      end
      
    else
      N_RUNS = n_run-1;
      if (n_run == 1)
        error_log_text("206: Model returned an error at first run (nominal values) - simulation stopped.");
        [str, n, line, func] = lasterror(%t);
        if (func == "") func = model_exec_file; end;        
        log_text( sprintf("&nbsp &nbsp Function: %s at line %d", func, line), 1,1 );
        log_text("&nbsp &nbsp Error: "+str, 1,1);
        ui_controls(ID_B_RUN).String= "Run Simulation";  
        ui_controls(ID_B_RUN).Callback = 'perform_monte_carlo_sim()';
        log_text("");
        return;
      end
      
      if (n_run == 2)
        error_log_text("207: Model returned an error at second run - simulation stopped.");
        [str, n, line, func] = lasterror(%t);
        if (func == "") func = model_exec_file; end;        
        log_text( sprintf("&nbsp &nbsp Function: %s at line %d", func, line), 1,1 );
        log_text("&nbsp &nbsp Error: "+str, 1,1);
        log_text("");
        ui_controls(ID_B_RUN).String= "Run Simulation";  
        ui_controls(ID_B_RUN).Callback = 'perform_monte_carlo_sim()';
        return;
      end
      
      error_log_text("208: Model returned an error at run "+ sprintf("%d", n_run)+" - simulation stopped.");
      [str, n, line, func] = lasterror(%t);
      if (func == "") func = model_exec_file; end;        
      log_text( sprintf("&nbsp &nbsp Function: %s at line %d", func, line), 1,1 );
      log_text("&nbsp &nbsp Error: "+str, 1,1);
          
      log_text("N_RUNS reduced to "+sprintf("%d", N_RUNS) + " - first valid results can be plotted and saved.",1,1);
      
      for i=1:1:out_variables(ID_OUTVAR_COUNT)  // save valid output variables       
          if (out_variables(ID_OUTVAR_TYPE)(i) == "scalar")
            model_output(i) = model_output(i)(1:N_RUNS);            
          end
          if (out_variables(ID_OUTVAR_TYPE) == "vector")
            
          end                      
      end
      
      for i=1:1:in_variables_statistics(ID_INVAR_COUNT)  // save valid input variables       
        model_input(i) = model_input(i)(1:N_RUNS);            
      end
      
      try
        xx = ui_controls(ID_WINDOW_LIST)(ID_PLOT_WINDOW);
        fill_variable_lists();
      catch
        ui_controls(ID_B_PLOT_MAIN).enable= "on";
      end      
  
      ui_controls(ID_B_RUN).enable= "on";
      ui_controls(ID_B_SAVE).enable= "on";
      ui_controls(ID_B_LOAD).enable= "on";
      ui_controls(ID_B_RUN).String= "Run Simulation";  
      ui_controls(ID_B_RUN).Callback = 'perform_monte_carlo_sim()';
      
      return;
    end
  else // vector model
    error_log_text("209: Model returned an error - simulation stopped. Nothing to recover from a vector model.");
  end

endfunction;




function [err] = call_model()

  global('n_run');  
  global('model_input');
  global('model_output');
  err = 0;
  if (model_type == "scalar")
    pre_run_scalar();
  else
    pre_run_vector();
  end
  
  for i=1:1:in_variables_statistics(ID_INVAR_COUNT)
    global(in_variables_statistics(ID_INVAR_NAME)(i));
  end

  for i=1:1:out_variables(ID_OUTVAR_COUNT)
    global(out_variables(ID_OUTVAR_NAME)(i));
  end
  
  err = exec(model_exec, 'errcatch');
  
  if (err~=0) return; end;
  
  if (model_type == "scalar")
    post_run();
  end
  
endfunction

function [] = pre_run_scalar()
  global('model_input');
  global('model_output');
  global('in_variables_statistics');
  global('n_run');
  
  if (in_variables_statistics(ID_INVAR_COUNT) > 0)
  for i=1:1:in_variables_statistics(ID_INVAR_COUNT)
    global(in_variables_statistics(ID_INVAR_NAME)(i));
    execfun = ""+in_variables_statistics(ID_INVAR_NAME)(i) + "= model_input"+sprintf("(%d)(%d)", i, n_run)+";";
    execstr(execfun); 
  end  
  end
endfunction

function [] = pre_run_vector()
  global('model_input');
  global('model_output');
  global('in_variables_statistics');
  global('n_run');
  if (in_variables_statistics(ID_INVAR_COUNT) > 0)
  for i=1:1:in_variables_statistics(ID_INVAR_COUNT)
    global(in_variables_statistics(ID_INVAR_NAME)(i));
    execfun = ""+in_variables_statistics(ID_INVAR_NAME)(i) + "= model_input"+sprintf("(%d)", i)+";";
    execstr(execfun); 
  end
  end

endfunction


function [] = post_run()
  global('model_input');
  global('model_output');
  global('out_variables');
  global('n_run');
  
  for i=1:1:out_variables(ID_OUTVAR_COUNT)
    global(out_variables(ID_OUTVAR_NAME)(i));
  end
  
  for i=1:1:out_variables(ID_OUTVAR_COUNT)  // assign result to output_vector
    execss = "model_output(i)(n_run) = "+ out_variables(ID_OUTVAR_NAME)(i) + ";" ;
    execstr(execss);  
  end
  
endfunction

function [] = post_all_runs(N_RUNS)
  global('model_input');
  global('model_output');
  global('out_variables');
    
  for i=1:1:out_variables(ID_OUTVAR_COUNT)
    global(out_variables(ID_OUTVAR_NAME)(i));
  end
  
  if (model_type == "vector")
    for i=1:1:out_variables(ID_OUTVAR_COUNT)  // assign result to output_vector
      execss = "model_output(i) = "+ out_variables(ID_OUTVAR_NAME)(i) + ";" ;
      execstr(execss);
    end
  end
endfunction
