
function [out] = read_spice_results(filename)
  ff = mopen(filename);
  foo = mgetl(ff,4);  // skip first lines
  [foo, foo, foo, varnum] = mfscanf(ff,"%s %s %d"); 
  [foo, foo, foo, tnum] = mfscanf(ff,"%s %s %d");  
  foo = mgetl(ff,3+varnum);  // skip lines  
  
  formatstring = "%s";
  for i=1:1:varnum
    formatstring = formatstring + "%f\n";
  end

  out = mfscanf(tnum, ff, formatstring);
  
  mclose(ff);  
endfunction

function [] = change_spice_component(cir_filename, component, value)
      
  spice_change_params = list(1);    
  spice_change_params(1) = [component;];
  spice_change_params(2) = value;
  change_ngspice_netlist(cir_filename, spice_change_params)  
  
endfunction

function [] = add_ngspice_line(spice_file, line_to_add)
  infile = mopen(spice_file, "r");
  s = mgetl(infile, -1);
  mclose(infile);
  
  outfile = mopen(spice_file, "w");
  for i=1:1:(size(s,1)-2)
    mfprintf(outfile, s(i)+"\n");     
  end
  
  mfprintf(outfile, line_to_add+"\n");
  mfprintf(outfile, s(size(s,1)-1)+"\n");
  mfprintf(outfile, s(size(s,1)  )+"\n");
    
  mclose(outfile);    
endfunction


function [] = change_ngspice_netlist(spice_file, spice_change_params);
	global("n_run");  

    [foo, err] = file ('open', spice_file+".mcir", 'unknown');
    if (err ~= 0)
        ifile = spice_file+".mcir";
        ofile = ifile;
    else
        ifile = spice_file;
        ofile = spice_file+".mcir";
    end
         
	infile = mopen(ifile, "r");
	s = mgetl(infile, -1);
	mclose(infile);
	count = 0;
	
	subckt = "";
	for i=1:1:size(s,1)
  
  		for j=1:1:size(spice_change_params(1), 1)
    		if (strindex(s(i), ".subckt")== 1)
      			[foo subckt] = sscanf(s(i), "%s %s");
	      		subckt = subckt+".";
    		end
  
    		if (strindex(subckt+s(i), spice_change_params(1)(j)+" ") == 1)
      			[foo1 foo2 foo3] = sscanf(s(i), "%s %s %s");
      			s(i) = foo1 + " " + foo2 + " " + foo3 + " " + spice_change_params(1+j);
      			count = count + 1;
    		end  	
  		end 	  
	end

	if (n_run == 1) 
  		montesci_console_info(sprintf("Changed %d spice netlist entries in ", count) +... 
  		"""" +shrink_string(spice_file, 50)+ """."); 
  	end

    outfile = mopen(ofile, "w");

for i=1:1:size(s,1)
  mfprintf(outfile, s(i)+"\n");     
end
mclose(outfile);
endfunction

function [] = change_spice_parameter(command_file, spice_change_params)
  
  try
    infile = mopen(command_file, "r");
    s = mgetl(infile, -1);
    mclose(infile);
  catch
    error("Error reading file """ + command_file+""""); 
  end
  try
  outfile = mopen(command_file+".mscn", "w");
  catch
    error("Error writing to file """ + command_file+".mscn""");
  end
  
  for i=1:1:max(size(s))
    
    if (strindex(s(i), "alter") ~= 1) mfprintf(outfile, s(i)+"\n");  end;    
    
    if (i==3)
      try  
      for j=1:1:size(spice_change_params(1), 1)
        ss = sprintf("alter %s [ ", spice_change_params(1)(j) );        
        ss = ss + sprintf(" %s ", spice_change_params(j+1) );
        ss = ss + "]";         
        mfprintf(outfile, ss+"\n");       
      end
      catch
        mclose(outfile);
        error("Error evaluating ngspice data change list supplied from your model.");
      end
    end
    
  end
  mclose(outfile);
  
endfunction

function ver = check_ngspice_ver()
  if (unix("ngspice --version") ~= 0) 
    ver = -1;
    return;
  end
  
  a = unix_g("ngspice --version");
  
  ver = "0";
  for i=1:1:size(a,1)
    x = strindex(a(i), "revision");  
    
    if (length(x) > 0)
      ver = part(a(1), x+9:x+10);  
    end
  end
  
  try
    execstr("ver = "+ver);
  catch
    ver = 0;
  end 
endfunction 


function [] = call_ngspice(netlist, command_file)
  
  if (force_ngspice ~= "on")  
  	if (ngspice_version) < 20
    	error("ngspice-20 or newer not found on your system, you are not able to run the model.");    
  	end
  end
  
  ret = unix_g("ngspice " + netlist +".mcir"+ " -i " + command_file);
  
  if (force_ngspice ~= "on")
  
    print_all = 0;
    for i=1:1:size(ret, 1)
    	if ( (length(strindex(ret(i), "Error")) > 0) & (print_all == 0) )
      		error_log_text ("215: Ngspice reported an error, output: ");
      		print_all = 1;            
    	end
    	if (print_all == 1)
      		log_text ("&nbsp &nbsp "+ret(i));
    	end
  	end
  
  	if (print_all == 1)
    	error("ngspice returned an error, above messages were returned.");
  	end
  end

endfunction




