
function [err] = prepare_variables (n_runs)
  global('model_input');
  global('model_output');
  global('in_variables_statistics');
  global('out_variables');
  
 
  log_text("Reading input_variables.conf ...", 1, 1);
  if (read_in_variables_conf()) err = %T; return; end;
      
  log_text("Preparing random variables for "+sprintf("%.d", n_runs) + " runs...", 1, 1);
  
  for i = 1:1:in_variables_statistics(ID_INVAR_COUNT)
    
    param1 = in_variables_statistics(ID_INVAR_PARAMETERS)(i)(1);
    param2 = in_variables_statistics(ID_INVAR_PARAMETERS)(i)(2);
    param3 = in_variables_statistics(ID_INVAR_PARAMETERS)(i)(3);
    param4 = in_variables_statistics(ID_INVAR_PARAMETERS)(i)(4);
    param5 = in_variables_statistics(ID_INVAR_PARAMETERS)(i)(5);
    lower_limit =  in_variables_statistics(ID_INVAR_LIMITS)(i)(1);
    upper_limit =  in_variables_statistics(ID_INVAR_LIMITS)(i)(2);
    

    
    if (upper_limit < lower_limit)
       log_text(sprintf("<B>Upper limit for variable '"%s'" is lower than lower limit - ignoring limits </B>", in_variables_statistics(ID_INVAR_NAME)(i)),1,1)
       upper_limit = lower_limit;
    end
    digits =  in_variables_statistics(ID_INVAR_DIGITS)(i);
    
    a_ind = (1:1:n_runs);
    model_input(i) = zeros(n_runs);
    
    count_iter = 0;  
    while (length(a_ind)>0)  // generate random variables with limits
          
      if (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "nor")                                      // gaussian normal distribution   
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'nor',param1 ,param2);   
      
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "bet")                                     // beta distribution
          if (param1 <=0) 
            error_log_text("301: Argument 1 (A) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end          
          if (param2 <=0) 
            error_log_text("301: Argument 1 (B) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end          
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'bet',param1 ,param2);
      
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "bin")                                     //binomial distribution
          if (param2 >1) | (param2 <0)
            error_log_text("301: Argument 2 (p) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" is out of bounds. ");
            err = %T; return;
          end
          if (param1 <1) 
            error_log_text("301: Argument 1 (N) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 1. ");
            err = %T; return;
          end          
          if ((round(param1) - param1)   ~=0)
            param1 = round(param1);
            log_text("<B>Argument 1 of input variable """ + in_variables_statistics(ID_INVAR_NAME)(i) +""" has been rounded.</B>", 1,1)
          end
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'bin',param1 ,param2);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "nbn")                                     //negative binomial distribution
          if (param2 >1) | (param2 <0)
            error_log_text("301: Argument 2 (p) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" is out of bounds. ");
            err = %T; return;
          end
          if (param1 <1) 
            error_log_text("301: Argument 1 (N) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be >= 1. ");
            err = %T; return;
          end          
          if ((round(param1) - param1)   ~=0)
            param1 = round(param1);
            log_text("<B>Argument 1 of input variable """ + in_variables_statistics(ID_INVAR_NAME)(i) +""" has been rounded.</B>", 1,1)
          end                                                                                               
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'nbn',param1 ,param2);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "chi")                                     // chi square distribution
          if (param1 <= 0) 
            error_log_text("301: Argument 1 (Df) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end             
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'chi',param1);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "nch")                                    // non central chi square distribution
          if (param1 <1) 
            error_log_text("301: Argument 1 (Df) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 1. ");
            err = %T; return;
          end             
          if (param2 <=0) 
            error_log_text("301: Argument 2 (Xnon) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end             
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'nch',param1, param2);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "exp")                                     // exponential distribution function
          if (param1 <0) 
            error_log_text("301: Argument 1 (Av) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be >= 0. ");
            err = %T; return;
          end             
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'exp',param1);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "f")                                       // F variance ratio
           if (param1 <=0) 
            error_log_text("301: Argument 1 (Dfn) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end             
          if (param2 <=0) 
            error_log_text("301: Argument 2 (Dfd) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end             
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'f',param1, param2);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "nf")                                      // non central F variance ratio
          if (param1 < 1) 
            error_log_text("301: Argument 1 (Dfn) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be >= 1. ");
            err = %T; return;
          end             
          if (param2 <=0) 
            error_log_text("301: Argument 2 (Dfd) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end             
          if (param3 <0) 
            error_log_text("301: Argument 3 (Xnonc) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be >= 0. ");
            err = %T; return;
          end             
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'nf',param1, param2, param3);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "gam")                                     // gamma distribution
          if (param1 <= 0) 
            error_log_text("301: Argument 1 (shape) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end             
          if (param2 <=0) 
            error_log_text("301: Argument 2 (scale) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be > 0. ");
            err = %T; return;
          end
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'gam',param1, param2);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "geom")                                    // geometric distribution
          if (param1 <= 0) | (param1 >1)
            error_log_text("301: Argument 1 (p) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be in the limits [1.3e-307, 1]. ");
            err = %T; return;
          end                   
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'geom',param1);

      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "poi")                                     // poisson distribution
          if (param1 < 0) 
            error_log_text("301: Argument 1 (mue) for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" must be >= 0. ");
            err = %T; return;
          end                   
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'poi',param1);

      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "unf")                                     // uniform distribution
          model_input(i)(a_ind) = grand(length(a_ind), 1, 'unf',param1, param2);
          
      elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "sweep")                                   // parameter sweep
          
          step = param2 - param1;
          step = step/(n_runs-1);
          vare = param1 : step : param2;
          
          model_input(i)(a_ind) = vare';
     
     elseif (in_variables_statistics(ID_INVAR_DISTRIBUTION)(i) == "const")                                   // constant
          model_input(i)(a_ind) = ones(length(a_ind), 1)  * strtod( in_variables_statistics(ID_INVAR_NOMINAL)(i));
        
      else
         error_log_text("302: Variable definition for '"" +sprintf("%s", in_variables_statistics(ID_INVAR_NAME)(i)) +"'": invalid distribution function");
         abort_execution();
      end 
      
      //check variable limits, if not met - recalculate not confined elements again
      if (upper_limit == lower_limit)   
        a_ind = [];        
      else        
        a_ind = [find(model_input(i)>upper_limit) find(model_input(i)<lower_limit)]; // check limits
      end // if end

      count_iter = count_iter + 1;
      
      if (modulo(count_iter,1000)==0 )
        log_text(sprintf("%d iterations to meet limits for '"%s'"... ", count_iter, in_variables_statistics(ID_INVAR_NAME)(i)),1,1)
      end
      
      if (count_iter == 10000)
        log_text(sprintf("<B>Skipping to meet limits for '"%s'" after 10000 iterations. </B>", in_variables_statistics(ID_INVAR_NAME)(i)),1,1)
        log_text(sprintf("<B>Variable '"%s'" will not be confined to its limits. </B>", in_variables_statistics(ID_INVAR_NAME)(i)),1,1)
        a_ind = [];
      end     
      
    end // while end
    
    
    // set nominal value
    if (in_variables_statistics(ID_INVAR_NOMINAL)(i)) == "average"
      model_input(i)(1) = mean(model_input(i));
    elseif (in_variables_statistics(ID_INVAR_NOMINAL)(i)) == "none"
        // no average value for this variable
    else
      if (isnum(in_variables_statistics(ID_INVAR_NOMINAL)(i)))
        model_input(i)(1) =strtod( in_variables_statistics(ID_INVAR_NOMINAL)(i));
      else
      
        error_log_text("303: Nominal value definition for  """+in_variables_statistics(ID_INVAR_NAME)(i)+""" is invalid (number, ""none"" or ""average"" are allowed). ");
        err = %T; return;
      end
    end
    
    
    // adjust digits
    err = digits - round(digits);
    if (err ~= 0)
      log_text(sprintf("Digit number for '"%s'" truncated to %d ", count_iter, in_variables_statistics(ID_INVAR_NAME)(i), digits),1,1)
    end
    digits = round(digits)
    model_input(i) = round(model_input(i) * (10^digits)) / 10^digits;

   
    // check if nominal values is within limits
    if (upper_limit ~= lower_limit)
      if (model_input(i)(1) > upper_limit) | (model_input(i)(1) < lower_limit)
        log_text("<B>Nominal value of input variable """+in_variables_statistics(ID_INVAR_NAME)(i)+""" is beyond specified limits. </B>",1,1);
      end 
    end


  end // for end
  
  log_text("Reading output_variables.conf ...", 1, 1);

  if (read_out_variables_conf()) err = %T; return; end;

  
  for i=1:1:out_variables(ID_OUTVAR_COUNT)
    if (out_variables(ID_OUTVAR_TYPE)(i) == "scalar")
      model_output(i) = zeros(n_runs, 1); // scalar output variables
    else
      model_output(i) = list(1);  // vector and matrices  
    end
  end  
  err = %F;
  
endfunction



ID_INVAR_NAME = 1;
ID_INVAR_COUNT = 2;
ID_INVAR_PARAMETERS = 3; 
ID_INVAR_UNIT = 4;
ID_INVAR_DISTRIBUTION = 5;
ID_INVAR_DIGITS = 6; 
ID_INVAR_LIMITS = 7;
ID_INVAR_NOMINAL = 8;

function [err] = read_in_variables_conf()
  global('in_variables_statistics');
  global('ui_controls');
  
  try
    conf_file = mopen(model_dir + invar_conf);
  catch
    error_log_text("304: File """+ invar_conf+""" could not be opened. ");
    log_text ("Look for: "+shrink_string(model_dir + invar_conf, 70),1,1);
    ui_controls(ID_B_RUN).enable= "on";
    ui_controls(ID_B_LOAD).enable= "on";
    err = %T;
    return;
  end
  
  conf_lines = stripblanks(mgetl(conf_file));

  distribution = [];
  name = [];
  parameters = list(1);
  unit = [];
  variable_count = 0;
  limits = list(1);
  var_digits = 0;
  nominal = [];
  
  for i=1:1:size(conf_lines,1)
    if ( (strncpy(conf_lines(i), 2) ~= "//") & (conf_lines(i) ~= "") )    // no comment in this line and no empty string
    variable_count = variable_count + 1;
  
    [n_param name(variable_count) unit(variable_count) distribution(variable_count) param1 param2 param3 param4 param5 ...
          var_digits(variable_count) lim_lower lim_upper nominal(variable_count)] = msscanf(conf_lines(i), "%s %s %s %f %f %f %f %f %f %f %f %s");

    if (n_param ~= 12)
      error_log_text(sprintf("305: Error parsing '"input_variables.conf'" line %d.", variable_count));
      err = %T;
      mclose(conf_file);
      return;
    end 
    parameters(variable_count)= [param1 param2 param3 param4 param5];
    limits(variable_count) = [lim_lower lim_upper];

    end
    err = %F;
  end

  mclose(conf_file);  
  
  // check variable names
  for i=1:1:variable_count
    [errc errtext] =check_var_name(name(i)) 
    if (errc == %t)
      err = %t;
      error_log_text(sprintf("305: Error parsing '"input_variables.conf'" line %d: ", i));
      log_text("&nbsp &nbsp "+errtext,1,1);
      return;     
    end
  end



  
  // check variable names uniqueness
  err_double = 0;
  for i=1:1:variable_count-1
    for j=(i+1):1:variable_count
    if (name(i) == name(j)) 
      err_double = 1;
      err=%t; 
      error_log_text(sprintf("305: Error parsing '"input_variables.conf'" line %d: ", i));
      log_text("&nbsp &nbsp " + sprintf("Variable ""%s"" has double definition lines. ", name(i)),1,1);
      mclose(conf_file);
      return;
      end;
    end
  end

  in_variables_statistics=list(1);
  in_variables_statistics(ID_INVAR_NAME) = name;
  in_variables_statistics(ID_INVAR_UNIT) = unit;
  in_variables_statistics(ID_INVAR_DISTRIBUTION) = distribution;
  in_variables_statistics(ID_INVAR_PARAMETERS) = parameters;
  in_variables_statistics(ID_INVAR_COUNT) = variable_count;
  in_variables_statistics(ID_INVAR_DIGITS)=var_digits;
  in_variables_statistics(ID_INVAR_LIMITS)=limits;
  in_variables_statistics(ID_INVAR_NOMINAL)=nominal;
  
  
endfunction

function [err, text] = check_var_name(var_name)
  // already defined?
  
  text = "";
  
  str = sprintf("y = %s", var_name);
  defined =1;
  
    if (execstr(str, 'errcatch') ~= 0)
      defined = 0;
    end

  
  if (defined == 1)
    text = "variable """+ var_name + """ is already defined in the scope of the model, use another name.";
  end 
  // valid name?
  
  str = sprintf("%s = 5", var_name);
  valid = 1;
  if (execstr(str, 'errcatch') ~= 0)
    valid = 0;
  end
  
  if (valid == 0)
    text="variable name """+ var_name + """ is invalid, use another name.";
  end 
  
  err = %f;
  if (valid ==0 ) | (defined == 1)
    err = %t;
  end
  
endfunction

  
ID_OUTVAR_NAME = 1;
ID_OUTVAR_UNIT = 2;
ID_OUTVAR_COUNT = 3;
ID_OUTVAR_LIMITS = 4;
ID_OUTVAR_TYPE = 5;
  
function [err] = read_out_variables_conf()
  global("out_variables");
  global('ui_controls');
    
  try
    conf_file = mopen(model_dir + outvar_conf);
  catch
    error_log_text("306: File """+ outvar_conf+""" could not be opened. ");
    log_text ("Look for: "+shrink_string(model_dir + outvar_conf, 70),1,1);
    ui_controls(ID_B_RUN).enable= "on";
    ui_controls(ID_B_LOAD).enable= "on";
    err = %T;
    return;
  end
  
  conf_lines = stripblanks(mgetl(conf_file));

  name = [];
  unit = [];
  variable_count = 0
  var_type = [];
  limits = list(1);

  for i=1:1:size(conf_lines,1)
    if ( (strncpy(conf_lines(i), 2) ~= "//") & (conf_lines(i) ~= "") )    // no comment in this line and no empty string
    variable_count = variable_count + 1;
    
    [n_param name(variable_count) unit(variable_count) ...
         lim_lower lim_upper var_type(variable_count)] = msscanf(conf_lines(i), "%s %s %f %f %s");
  
    if (n_param ~= 5)
      error_log_text(sprintf("307: Error parsing '"output_variables.conf'" line %d.", variable_count));
      err = %T;
      mclose(conf_file);
      return;
    end 
  
    limits(variable_count)  = [lim_lower lim_upper];
    end
  end
    
  

  // check variable names validity
  for i=1:1:variable_count
    [errc errtext] =check_var_name(name(i)) 
    if (errc == %t)
      err = %t;
      error_log_text(sprintf("307: Error parsing '"output_variables.conf'" line %d: ", i));
      log_text("&nbsp &nbsp "+errtext,1,1);
       mclose(conf_file);
      return;     
    end
  end

  // check variable names uniqueness
  err_double = 0;
  for i=1:1:variable_count-1
    for j=(i+1):1:variable_count
    if (name(i) == name(j)) 
      err_double = 1;
      err=%t; 
      error_log_text(sprintf("307: Error parsing '"output_variables.conf'" line %d: ", i));
      log_text("&nbsp &nbsp " + sprintf("Variable ""%s"" has double definition lines. ", name(i)),1,1);
      mclose(conf_file);
      return;
      end;
    end
  end
  
  out_variables(ID_OUTVAR_NAME) = name;
  out_variables(ID_OUTVAR_UNIT) = unit;
  out_variables(ID_OUTVAR_COUNT) = variable_count;
  out_variables(ID_OUTVAR_LIMITS) = limits;
  out_variables(ID_OUTVAR_TYPE) = var_type;

  mclose(conf_file);
  
endfunction



  