ID_NEW_SIM_WINDOW = 301;
ID_T_NEW_SIM_PATH = 310;
ID_B_NEW_SIM_CREATE = 311;
ID_B_NEW_SIM_CANCEL = 312;
ID_B_NEW_SIM_EDIT_PATH = 313;
ID_E_NEW_SIM_PATH = 314;
ID_TT_NEW_SIM_PATH = 315;

function [] = create_new_sim_gui()
  global('ui_controls');

  gui_main = figure(ID_NEW_SIM_WINDOW);
  gui_new_sim=gcf();
  ui_controls(ID_WINDOW_LIST)(ID_NEW_SIM_WINDOW) = gui_new_sim;
  gui_main.position = [20 20 600 90];
  
  gui_main.figure_name = "Create new Montesci directory";
  gui_main.background = gui_color_bg;
  sleep(200);
  toolbar(ID_NEW_SIM_WINDOW, 'off');
  
  delmenu(ID_NEW_SIM_WINDOW, 'File');
  delmenu(ID_NEW_SIM_WINDOW, 'Edit');
  delmenu(ID_NEW_SIM_WINDOW, 'Tools');
  delmenu(ID_NEW_SIM_WINDOW, '?');
  sleep(200);
  ui_controls(ID_E_NEW_SIM_PATH) = uicontrol(gui_new_sim, 'style', 'edit', 'position', [20 100 300 24], 'String', "Enter_name_here");
  
  pmstr = "Create in: " +  pwd();
  ui_controls(ID_T_NEW_SIM_PATH) = uicontrol(gui_new_sim, 'style', 'text', 'position', [20 60 420 24], 'String', pmstr);
  ui_controls(ID_T_NEW_SIM_PATH).BackgroundColor =  gui_plot_input_bg;
  
  
  ui_controls(ID_B_NEW_SIM_CREATE) = uicontrol(gui_new_sim, 'style', 'pushbutton', 'position', [470 20 110 28], 'String', "Create", 'Callback', 'new_sim_directory();');
  ui_controls(ID_B_NEW_SIM_CANCEL) = uicontrol(gui_new_sim, 'style', 'pushbutton', 'position', [20 20 110 28], 'String', "Cancel", 'Callback','');
  ui_controls(ID_B_NEW_SIM_EDIT_PATH) = uicontrol(gui_new_sim, 'style', 'pushbutton', 'position', [470 58 110 28], 'String', "Change...", 'Callback','change_pwd()');
  ui_controls(ID_TT_NEW_SIM_PATH) = pwd();
  
endfunction

function [] =change_pwd()
  global('ui_controls');
  this_dir = pwd();
  directory = uigetdir();
  if (directory == "")
    return; 
  end
  try
    chdir(directory);
    pmstr = "Create in: " +  pwd();
    ui_controls(ID_T_NEW_SIM_PATH).String = pmstr;
    ui_controls(ID_TT_NEW_SIM_PATH) = pwd();
    chdir(this_dir);
  catch
    messagebox("Could not change directory, do you have read permissions?.", "Error", "error", ["Continue"], "modal")
  end
  chdir(this_dir);
endfunction
 

function []  = new_sim_directory()
  global('ui_controls');
  dir_short = ui_controls(ID_E_NEW_SIM_PATH).String;
  directory = ui_controls(ID_TT_NEW_SIM_PATH) +  "/" +dir_short;
  res =  mkdir(directory);
  
  if ( res == 0 ) // could not write
    messagebox("Could not create directory, do you have write permissions?.", "Error", "error", ["Continue"], "modal")
    return;
  end
  
  function [result] = montesci_dir_init()
    result = copyfile(montesci_root+"macros/template/input_variables.conf" ,directory+"/input_variables.conf");
    result = result + copyfile(montesci_root+"macros/template/model.conf" ,directory+"/model.conf");
    result = result + copyfile(montesci_root+"macros/template/model_pre_exec.sce" ,directory+"/model_pre_exec.sce");
    result = result + copyfile(montesci_root+"macros/template/model.sce" ,directory+"/model.sce");
    result = result + copyfile(montesci_root+"macros/template/output_variables.conf" ,directory+"/output_variables.conf");
    result = result - 5;
  endfunction
  
  if (res == 1)  // success creating directory
    res1 = montesci_dir_init();
    if (res1 <> 0)
      messagebox("Could not copy necessary files.", "Error", "error", ["Continue"], "modal")
      return;
    end

    close(ui_controls(ID_WINDOW_LIST)(ID_NEW_SIM_WINDOW));
    log_text("Created Montesci simulation directory """+ dir_short +""".", 1,1);
    global('model_dir');
    model_dir = directory+"/";
    log_text("Current model directory:  """+ shrink_string(model_dir, 60) +""".", 1,1);
         
  end
  
  if (res == 2)  // directory exists
    ans = messagebox("Directory already exists. Continue copying files? Existing files will be overwritten.", "Error", "error", ["Copy files"; "Cancel"], "modal");
    if (ans ==1)      
      res1 = montesci_dir_init();
      if (res1 <> 0)
      messagebox("Could not copy necessary files.", "Error", "error", ["Continue"], "modal")
      return;
    end    
      close(ui_controls(ID_WINDOW_LIST)(ID_NEW_SIM_WINDOW));
      log_text("Created Montesci simulation directory """+ dir_short +""".", 1,1);
      model_dir = directory+"/";
      log_text("Current model directory:  """+ shrink_string(model_dir, 60) +""".", 1,1);
    end
    global('model_dir');
    model_dir = directory+"/";
    
  end
endfunction


function update_model_dirs()
  global("model_dir_last");
  global("model_dir");
  
  if (model_dir_last(1) ~= model_dir)  
    for i=6:-1:2
      model_dir_last(i) = model_dir_last(i-1);
    end  
  end
  
  model_dir_last(1) = model_dir;
  
  try
    save(montesci_root+"/montesci.last", model_dir_last); // save current model directory
  catch
    log_text("Warning: Could not save model history file.", 1,1);
  end
  
endfunction


function [] = open_sim_directory_d(directory)    
    global('model_dir');
    if (directory == "---") return; end;    
    if (length(model_dir) > 3)         
      update_model_dirs();
      update_model_dir_menu();  
    end
    
    try
      ff = mopen(directory+"model.conf", "r");
      mclose(ff);
      log_text("Current model directory:  """+ shrink_string(directory, 60) +""".", 1,1);
      model_dir = directory;      
    catch
      error_log_text("104: Directory:  """+ shrink_string(directory, 40) +"""could not be opened.");
      model_dir = "";
      return;
    end
      
endfunction


function [] = open_sim_directory()
  
  directory = uigetdir();
  this_dir = pwd();
  if (directory == "")
    return; 
  end
  try
    chdir(directory);
    dir_long = pwd();
    chdir(this_dir);
    f = mopen(directory + "/model.conf");
    mclose(f);
  catch
    messagebox("Could not find a model.conf there, sure that is a MonteSci directory?", "Error", "error", ["Continue"], "modal")
    return;
  end
  
  open_sim_directory_d(dir_long+"/");
      
endfunction;



function [] = open_demo_directory(nr)
  model_dir = montesci_root+"/demos/demo_model_"+sprintf("%d",nr)+"/";
  this_dir = pwd();
  try
    chdir(model_dir);
    dir_long = pwd();
    chdir(this_dir);
    f = mopen(dir_long + "/model.conf");
    mclose(f);
  catch
    messagebox("Could not find a model.conf there - something wrong with the path of the demo?", "Error", "error", ["Continue"], "modal")
    return;
  end
  open_sim_directory_d(dir_long+"/");  
    
endfunction
