// MonteSci, monte carlo simulator for scilab
// ka_eins, 06/2011
// main script, execute this script for program execution

funcprot(0);
stacksize("max");

function result = test_montesci_call()
  result = %F;
  
  hist = gethistory(); 
  hist = hist(max(size(hist)));
  
  if (strindex(hist, "exec")  == []) result = %T;
  end  
endfunction

// check call, montesci has to be executed as macro

if (~isdef("test_montesci"))
  if (test_montesci_call())
    disp("MonteSci cannot be executed as a function. Please execute as a macro by ""exec(montesci)""");
    return;
  end
end

// model settings

model_dir = "";
global('model_dir');

if (~exists("montesci_root"))
  montesci_root = get_absolute_file_path("montesci.sce");
  montesci_root = montesci_root + "../";
end


// checking version of scilab (required > 5.2.0)

valid_scilab_versions = ["5.2.0"; "5.2.1"; "5.2.2"; "5.3.0"; "5.3.2"; "5.3.3"];
scilab_version = part(getversion(), 8:length(getversion()));
startup_exec = %F;
N_versions = size(valid_scilab_versions);
N_versions = N_versions(1);

versions_string = "";

for i=1:1:N_versions
  if (part(scilab_version,1:5) == valid_scilab_versions(i))
    startup_exec = %T;    
  end
  versions_string = versions_string + valid_scilab_versions(i) + "  ";
end

if (startup_exec == %F)

  disp("ERROR: MonteSci requires one of the following Scilab versions: ");
  disp(valid_scilab_versions);
  if (scilab_version == "3.0") return; end;
  if (scilab_version == "3.1.1") return; end;
  if (scilab_version == "4.0") return; end;
  if (scilab_version == "4.1") return; end;
  if (scilab_version == "4.1.1") return; end;
  if (scilab_version == "4.1.2") return; end;
    
  messagebox("<HTML> <BODY> <B>MonteSci was not (yet) tested with your Scilab version.</B><BR> Please use one of these versions:<BR>" + versions_string + "</BODY></HTML>"  , "Error", "error", ["Continue"], "modal");
  if (scilab_version == "5.0") return; end;
  if (scilab_version == "5.0.1") return; end;
  if (scilab_version == "5.0.2") return; end;
  if (scilab_version == "5.0.3") return; end;
  if (scilab_version == "5.1") return; end;
  if (scilab_version == "5.1.1") return; end; 

  disp("Your version seems to be newer than 5.2.0? Why not try this verion - but be prepared for issues");

end

// init random generator
dt=getdate();
n = dt(length(dt));
grand('setsd',n)

// program information
montesci_version = "1.21";

//montesci_root = get_absolute_file_path("montesci.sce");
force_run = "off";  // "on" ignores scilab version 


// simulation configuration
N_SIM_MAX = 500001; // limit maximum number of runs set in model.conf

// main gui configuration
gui_color_bg = color(0,0,0);
close;
gui_color_bg_log = [0.95, 1, 0.9];
gui_color_bg_2 = [0.95 0.9 1];

N_LOG_WINDOW = 40;

// plot gui configuration
gui_plot_input_bg = [0.95, 1, 0.9];
//gui_plot_input_bg = [193/255, 255/255, 1];
gui_plot_output_bg = [0.95, 0.9, 1];
gui_plot_disabled_text = [0.7 0.7 0.7];
gui_plot_enabled_text = [0 0 0];
gui_plot_hstplt_clscnt = [2 5 10 20 50 100 200 500 1000];
N_dec_3d =  4000;
N_plot_3d_max = 400;

// plot window configurations
xy_fit_fkt_max_points = 2000;


// Load Montesci

exec(montesci_root+"macros/montesci_helper.sce");

exec(montesci_root+"macros/plot/plot_main.sce");
exec(montesci_root+"macros/plot/plot_2d.sce");
exec(montesci_root+"macros/plot/plot_3d.sce");
exec(montesci_root+"macros/plot/plot_histogram.sce");
exec(montesci_root+"macros/plot/plot_surface.sce");
exec(montesci_root+"macros/plot/plot_wave.sce");

exec(montesci_root+"macros/gui/gui_main.sce");
exec(montesci_root+"macros/gui/gui_plot.sce");
exec(montesci_root+"macros/gui/gui_variable_explorer.sce");
exec(montesci_root+"macros/gui/gui_create_new.sce");
exec(montesci_root+"macros/gui/gui_fit_functions.sce");

exec(montesci_root+"macros/exec/log_functions.sce");
exec(montesci_root+"macros/exec/load_save_simulation.sce");
exec(montesci_root+"macros/exec/abort_pause_execution.sce");
exec(montesci_root+"macros/exec/model_execution.sce");
exec(montesci_root+"macros/exec/variable_preparation.sce");
exec(montesci_root+"macros/exec/ngspice_functions.sce");
exec(montesci_root+"macros/exec/post_run_exec.sce");


model_input = list(1);
global('model_input');

model_output = list(1);
global('model_output');

n_run = 0;
global('n_run');

in_variables_statistics = list(1);
global('in_variables_statistics');

out_variables = list(1);
global("out_variables");


invar_conf = model_dir + "input_variables.conf"; 
outvar_conf = model_dir + "output_variables.conf";
sim_conf = model_dir + "model.conf";

disp("This is MonteSci "+ montesci_version); 

main_gui = create_montesci_gui();

ngspice_version = check_ngspice_ver();
if (ngspice_version > 0)
  if (ngspice_version > 19)
    disp("  INFO: ngspice >= 20 found, you will be able to run ngspice models."); 
  end
  if (ngspice_version < 20)
    disp("  WARNING: Your ngspice installation is too old , you will NOT be able to run ngspice models."); 
  end
  if (ngspice_version == 0)
    disp("  WARNING: ngspice found, but cannot determine version, assume 20.");
    ngspice_version = 20; 
  end
  if (ngspice_version == -1)
    disp("  WARNING: ngspice not found, you will not be able to run ngspice models"); 
  end
    
end

exec(montesci_root+"montesci.conf");


// fit function definition
global('fit_functions');

fit_functions = list(1);

ff1=list(1);
ff1(1) = "y = f(x, c);";
ff1(2) = "c0(1), c0(2), ....";
ff1(3) = "New fit function";
ff1(4) = "";
fit_functions(1) = ff1;

ff1=list(1);
ff1(1) = "y = c(2)*x +c(1);";
ff1(2) = "0, 0";
ff1(3) = "Linear fit";
ff1(4) = "locked";
fit_functions(2) = ff1;
clear ff1;

ff1=list(1);
ff1(1) = "y = c(3)*x**2+c(2)*x + c(1);";
ff1(2) = "0, 0, 0";
ff1(3) = "Quadratic fit";
ff1(4) = "locked";
fit_functions(3) = ff1;
clear ff1;

ff1=list(1);
ff1(1) = "y = c(4)*x**3 + c(3)*x**2 + c(2)*x + c(1);";
ff1(2) = "0, 0, 0, 0";
ff1(3) = "Cubic fit";
ff1(4) = "locked";
fit_functions(4) = ff1;
clear ff1;

model_version = "unknown";





