
function [] = montesci_plot2d(xdata, ydata, xlab, ylab, tit, fit_fkt, valid_runs)

  scf;
  plot(xdata, ydata, "black.");
  a=gca();
  append_plot_to_list(a.parent);
  
  a.children(1).children.mark_size=2;
  
  plot(xdata(1), ydata(1), "magenta x");
  a.children(1).children.mark_size=20;
  
  a.parent.event_handler = "xyplot_handler";
  a.parent.event_handler_enable = "on";
  
  xgrid;
  xlabel(xlab)
  ylabel(ylab);
  title(tit)

  a.parent.background = -2;
  
  a.parent.color_map(33,:) = [0.95 0.9 1];
  a.parent.background = 33;
  a.children.children(1).thickness = 2;
  a.title.font_size = 3;
  a.title.font_style=5;
  a.x_label.font_style = 6;
  a.x_label.font_size = 2;
  a.y_label.font_style = 6;
  a.y_label.font_size = 2;
  a.foreground = 2;
  
  if (fit_fkt ~= [])
  
    fs = a.parent.figure_size;
    sizex= 200;
    sizey= 30;
    x= fs(1) * 0.5-sizex/2; 
    y= fs(2) * 0.5-sizey/2; 
    
    b=uicontrol(a.parent, "Style","text","String","Calculating fit function...", "Position", [x y sizex sizey]);
    b.visible = "on";
    b.Relief = "groove"
    b.BackgroundColor = gui_color_bg_log;
    b.HorizontalAlignment = "center";
  
    s_exec = "start_val = [" + fit_fkt(2) + "]''";
    try
      execstr(s_exec);
    catch
      b.String = "<HTML><B>Error calculating fit function.</B></HTML>";
      return;
    end

    [x, y, err, c] = make_fit(xdata, ydata, fit_fkt(1), start_val);

    if (err==0)
      b.visible = "off";
      plot(x,y,"green");
      a.children.children(1).thickness = 2;

      formula_string = replace_cs(strip_all_blanks(fit_fkt(1)), c);
      
      fs = a.parent.figure_size;
      sizex= 300;
      sizey= 50;
      x= fs(1) * 0.6-sizex/2; 
      y= fs(2) * 0.15-sizey/2; 
    
      b=uicontrol(a.parent, "Style","text","String",formula_string, "Position", [x y sizex sizey]);
      b.visible = "on";
      b.Relief = "groove"
      b.BackgroundColor = gui_color_bg_log;
      b.HorizontalAlignment = "left";
      
      
    else
      b.String = "<HTML><B>Error calculating fit function.</B></HTML>";    
    end

  end

  pick_text=uicontrol(a.parent, 'style', 'text', 'position', [100,100,100,28], 'String', '');
  pick_text.Visible = "off";
  pick_text.Relief = "groove"
  pick_text.BackgroundColor = gui_color_bg_log;
  pick_text.HorizontalAlignment = "center";  
  
  a.parent.user_data=list(1);
  a.parent.user_data(1) = valid_runs;
  
endfunction


function [] = xyplot_handler(win, xcl, ycl, ibut)
  
  if (ibut==3)
  
    global("N_RUNS");     
  
    a = get_figure_handle(win);
    plot_axes = a.children(size(a.children,1));
        
    valid_indices = a.user_data(1);
        
    [sz] = a.axes_size;   
    a.children(1).Visible = "off";  // turn picker off 
        
    x = (xcl-0.125*sz(1))/(sz(1)*0.75);    
    y = 1- (ycl-0.125*sz(2))/(sz(2)*0.75);
    
    x_labels = plot_axes.x_ticks.labels;
    x_start = x_labels(1);
    x_end = x_labels(max(size(x_labels)));
            
    exes = "x_start = "+x_start;
    execstr(exes);
    exes = "x_end = "+x_end;
    execstr(exes);
    
    y_labels = plot_axes.y_ticks.labels;
    y_start = y_labels(1);
    y_end = y_labels(max(size(y_labels)));
    
    exes = "y_start = "+y_start;
    execstr(exes);
    exes = "y_end = "+y_end;
    execstr(exes);
        
    x= x_start + (x_end-x_start) * x;
    y= y_start + (y_end-y_start) * y;

    data = plot_axes.children(size(plot_axes.children,1)).children(1).data;
    
    data(:,1) =abs(data(:,1) - x). / (x_end-x_start); 
    data(:,2) =abs(data(:,2) - y)./ (y_end  - y_start);
        
    diff = data(:,1) + data(:,2);
    
    [foo index_min] = min(diff);
     
    a.children(1).Position = [xcl-35 sz(2)-ycl+5 70 20];
    a.children(1).Visible = "on";
    a.children(1).String = sprintf("Run: %d", valid_indices(index_min)); 
    
    data = plot_axes.children(size(plot_axes.children,1)).children(1).data;
    
    a = plot_axes;
    
    a = sca(plot_axes); 
    
    if (a.children(1).user_data == "mark")
      a.children(1).visible="off"
    end
          
    plot(data(index_min,1),data(index_min,2), "*red");
     
    a.children(1).user_data = "mark";
        
    global("slider_ver");
    global("expl_sort_index");
    try    
      slider_ver.Value = min(valid_indices(index_min), slider_ver.Max);      
      expl_sort_index = 0;
      fill_values([valid_indices(index_min)]);
    catch
    end
    
  end
endfunction


function [x, y, errfit, c] = make_fit(data_x, data_y, fitfunctionstr, c0)
  
  function y=FF(x,c)
    try
      execstr(fitfunctionstr);
    catch
      error(22);
    end
  endfunction
  
  data_x_fit = decimate(data_x, xy_fit_fkt_max_points);
  data_y_fit = decimate(data_y, xy_fit_fkt_max_points);
    
  [foo, index] = gsort(data_x_fit, 'g', 'i');
  
  data_y_fit = data_y_fit(index);
  data_x_fit = data_x_fit(index);
  
  Z=[data_y_fit'; data_x_fit'];
    
  function e = minfun(c, data_x_fit, data_y_fit)
    e =  FF(data_x_fit, c) - data_y_fit; 
  endfunction
  
  errfit = 0;
  try
    [f, c, gropt] = leastsq(list(minfun,data_x_fit,data_y_fit),c0)
  catch
    errfit = 1;
  end
  
  stepi = ( max(data_x_fit) - min(data_x_fit) )/1000;
   
  x=min(data_x):stepi:max(data_x);
  
  y = FF(x,c);

endfunction