function [fit_fkt, param, param_name] = dist_fit(data, distfkt, classes, N_RUNS)
  xmin = min(data);
  xmax = max(data);
  step_x = (xmax - xmin) / 1000; 
  X = xmin : step_x : xmax;
  h = (max(data) - min(data))/classes;
  step_hist = (xmax - xmin) / classes;
  p1 = 0;
  p2 = 0;
  p3 = 0;
  p4 = 0;
  if (distfkt == 2) // normal
    p1 = mean(data);
    p2 = sqrt(variance(data));  
    Y = N_RUNS*step_hist * 1./sqrt(2*%pi*p2^2) .* exp((-(X-p1).^2)./(2*p2^2));
    param_name = ["mean: " ; "Standard deviation: "];
  end
  
  if (distfkt == 3) // beta
    m = mean(data);
    s = variance(data);  
    p1 = m*( m*(1-m)/s - 1);
    p2 = (1-m)*( m*(1-m)/s -1 );
    Y = N_RUNS*step_hist* 1/beta(p1, p2) .* X.^(p1-1).*(1-X).^(p2-1);
    param_name = ["A (shape): " ; "B (shape): "];
  end
  
  if (distfkt == 4) // chi square
    p1 = round(mean(data));
    Y = N_RUNS*step_hist * 1/(2**(p1/2) * gamma(p1/2)) .* X**(p1/2-1) .* exp(-X/2);
    param_name = ["Degrees of Freedom: "];
  end
  
  if (distfkt ==5)  // exp
    p1 = 1/mean(data);
    Y = N_RUNS*step_hist * p1 * exp(-p1.*X);
    index = find(X<0);
    Y(index) =0; 
    param_name = ["Av: "];
  end
  
  if (distfkt ==6) // gamma
    m = mean(data);
    s = variance(data);  
    p1 = m^2/s;
    p2 = m/p1;
    
    Y = N_RUNS*step_hist * X**(p1-1) .* exp(-X/p2) ./ (gamma(p1) *p2**p1);
    param_name = ["shape: " ; "scale: "];
  end
  
  fit_fkt = [X; Y];
  param = [p1 p2 p3, p4];
endfunction


function [] = montesci_histplot(classes, data, xlab, ylab, tit, lower_limit, upper_limit, fit_function, plot_nominal, valid_runs);
  
  global("N_RUNS");
  scf;

  histplot(classes, data,  normalization=%f);
 
  a=gca();
  append_plot_to_list(a.parent);
  xgrid;
  xlabel(xlab);
  ylabel(ylab);
  title(tit);
    
  a.parent.event_handler = "histplot_handler";
  a.parent.event_handler_enable = "on";
  
  if (lower_limit ~= upper_limit)
    xlu = [lower_limit-+1e-100 lower_limit+1e-100]
    ylu = [min(a.children.children.data(:,2)) max(a.children.children.data(:,2))] * 1.05;
    plot(xlu, ylu, "red"); 
    a.children.children(1).thickness = 2;
    
    xlu = [upper_limit-+1e-100 upper_limit+1e-100]
    plot(xlu, ylu, "cyan"); 
    a.children.children(1).thickness = 2; 
  
    y = (data > lower_limit) & (data < upper_limit);
    count_yield = sum(y);

    // plot nominal value 
    if (plot_nominal == %T)
      xlu = [data(1)-+1e-100 data(1)+1e-100]
      plot(xlu, ylu, "magenta"); 
      a.children.children(1).thickness =2;
    end 
  
  else  
    // plot nominal value
    if (plot_nominal == %T)
      xlu = [data(1)-+1e-100 data(1)+1e-100]
      ylu = [min(a.children(1).children.data(:,2)) max(a.children(1).children.data(:,2))]
      plot(xlu, ylu, "magenta"); 
      a.children.children(1).thickness = 2;
    end 
   
  end
  
  a.parent.background = -2;
  a.parent.figure_name = "MonteSci histogram plot";
  a.parent.color_map(33,:) = gui_color_bg_log;
  a.parent.background = 33;
  a.children.children.fill_mode="on";
  a.children.children.background=2;

  a.title.font_size = 3;
  a.title.font_style= 8;
  a.x_label.font_style = 6;
  a.x_label.font_size = 2;
  a.y_label.font_style = 6;
  a.y_label.font_size = 2;
  
  nom_val = sprintf("Nominal value: %.3E ", data(1));
  
  if (upper_limit ~= lower_limit)
    yield = sprintf("Yield: %d / %d, = %.3f percent", count_yield, length(data), count_yield/length(data)*100);
    uplim = sprintf("Upper limit: %d", upper_limit);
    lolim = sprintf("Lower limit: %d", lower_limit);
    legend(yield, lolim, uplim, nom_val);
  end
  
  // plot distribution functions
  
  if (upper_limit ~= lower_limit)
    xmin = min(lower_limit, min(data));
    xmax = max(upper_limit, max(data));
    
  else
    xmin = min(data);
    xmax = max(data);
  end

  if (fit_function > 1) // plot fit function as well
    
    fs = a.parent.figure_size;
    sizex= 200;
    sizey= 50;
    x= fs(1) * 0.68-sizex/2; 
    y= fs(2) * 0.15-sizey/2; 
    b=uicontrol(a.parent, "Style","text", "Position", [x y sizex sizey]);
    
    b.visible = "on";
    b.Relief = "groove"
    b.BackgroundColor = gui_color_bg_log;
    b.HorizontalAlignment = "left";
    
    try
      [fit_fkt, params, param_name] = dist_fit(data, fit_function, classes, N_RUNS)
    catch
      b.String = "<HTML><B>Error calculating fit function.</B></HTML>";
      return;
    end
    plot(fit_fkt(1,:),fit_fkt(2,:), "red");
    a.children.children(1).thickness = 2;

    distname = [""; "Normal"; "Beta"; "Chi square"; "Exponential"; "Gamma"];
                
    b.String = "<HTML><B>" + distname(fit_function) + " distribution parameters:</B>";
    b.String = b.String + "<BR>" + param_name(1) + sprintf("%.3E", params(1));
    sz = size(param_name);   
    if (sz(1)>1) b.String = b.String + "<BR>" + param_name(2)  +sprintf("%.3E", params(2));  end;
    b.String = b.String + "</HTML>";

  end
  a.parent.user_data=list(1);
  a.parent.user_data(1) = data;
  a.parent.user_data(2) = valid_runs;

endfunction


function [] = histplot_handler(win, xcl, ycl, ibut)
  if (ibut==3)
    a = get_figure_handle(win);
    plot_axes = a.children(max(size(a.children)));
        
    [sz] = a.axes_size;   
    //a.children(1).Visible = "off";  // turn picker off 
        
    x = (xcl-0.125*sz(1))/(sz(1)*0.75);    
    
    x_labels = plot_axes.x_ticks.labels;
    x_start = x_labels(1);
    x_end = x_labels(max(size(x_labels)));
    
    exes = "x_start = "+x_start;
    execstr(exes);
    exes = "x_end = "+x_end;
    execstr(exes);
    
    x= x_start + (x_end-x_start) * x;
    
    yy = max(size(plot_axes.children));
    
    data = plot_axes.children(yy).children(1).data;
    data = data(:,1);
        
    uindex = find(data >= x);
    lindex = find(data <= x);
 
    uindex = uindex(1);
    lindex = lindex(length(lindex));
    
    sdata = a.user_data(1);
    valid_indices = a.user_data(2);
    
    if (uindex ==[]) uindex = data(length(data)); end;
    if (lindex ==[]) lindex = 1; end;
    
    indices = find((sdata <= data(uindex)) & (sdata >= data(lindex)));
        
    indices = valid_indices(indices);
        
    global("slider_ver");
    try    
      slider_ver.Value = min(min(indices), slider_ver.Max);
      fill_values(indices);
    catch
    end
    //disp(data(uindex), data(lindex))
  end
endfunction
