
function []=append_plot_to_list(gcf_plt)
    global('ui_controls');  

    ui_controls(ID_PLOT_LIST)(length(ui_controls(ID_PLOT_LIST))+1) = gcf_plt;

endfunction



function outlist = reduce_valid_runs(in_list, valid_runs)

    if (length(in_list(1))>1) then //vector
        outlist=list(1);
        for i=1:1:length(valid_runs)
            outlist(i) = in_list(valid_runs(i));
        end
    else    //scalar
        outlist = in_list(valid_runs);
    end

endfunction

function [] = montesci_plot()

    global('model_input');
    global('model_output');
    global('ui_controls');
    global('N_RUNS');

    ui_controls(ID_B_PLOT).Enable ="off";

    [x_list y_list z_list] = get_xyz_list_selection();

    if (ui_controls(ID_PM_RUN_SEL).Value == 2 ) then
        valid_runs  = get_valid_indices(1:1:N_RUNS);
    else
        valid_runs  = 1:1:N_RUNS;
    end

    if (x_list(1) > 0)
        xdata = model_input(x_list(1)); 
        xdata = reduce_valid_runs(xdata, valid_runs);   
        x_name = in_variables_statistics(ID_INVAR_NAME)(x_list(1));
        x_unit = in_variables_statistics(ID_INVAR_UNIT)(x_list(1));
        x_upper_limit = in_variables_statistics(ID_INVAR_LIMITS)((x_list(1)))(2);
        x_lower_limit = in_variables_statistics(ID_INVAR_LIMITS)((x_list(1)))(1);    
        plot_nominal = in_variables_statistics(ID_INVAR_NOMINAL)(x_list(1));
        plot_nominal = (plot_nominal~="none");           
    end

    if (x_list(2) > 0)
        xdata = model_output(x_list(2))
        xdata = reduce_valid_runs(xdata, valid_runs);   
        x_name = out_variables(ID_OUTVAR_NAME)(x_list(2));
        x_unit = out_variables(ID_OUTVAR_UNIT)(x_list(2));
        x_upper_limit = out_variables(ID_OUTVAR_LIMITS)((x_list(2)))(2);
        x_lower_limit = out_variables(ID_OUTVAR_LIMITS)((x_list(2)))(1);
        plot_nominal = %T; 
    end

    if (y_list(1) > 0)
        ydata = model_input(y_list(1)); 
        ydata = reduce_valid_runs(ydata, valid_runs); 
        y_name = in_variables_statistics(ID_INVAR_NAME)(y_list(1));
        y_unit = in_variables_statistics(ID_INVAR_UNIT)(y_list(1));
        y_upper_limit = in_variables_statistics(ID_INVAR_LIMITS)((y_list(1)))(2);
        y_lower_limit = in_variables_statistics(ID_INVAR_LIMITS)((y_list(1)))(1);
    end
    if (y_list(2) > 0)
        ydata = model_output(y_list(2));
        ydata = reduce_valid_runs(ydata, valid_runs);   
        y_name = out_variables(ID_OUTVAR_NAME)(y_list(2));
        y_unit = out_variables(ID_OUTVAR_UNIT)(y_list(2));  
        y_upper_limit = out_variables(ID_OUTVAR_LIMITS)((y_list(2)))(2);
        y_lower_limit = out_variables(ID_OUTVAR_LIMITS)((y_list(2)))(1);
    end

    if (z_list(1) > 0)
        zdata = model_input(z_list(1)); 
        zdata = reduce_valid_runs(zdata, valid_runs); 
        z_name = in_variables_statistics(ID_INVAR_NAME)(z_list(1));
        z_unit = in_variables_statistics(ID_INVAR_UNIT)(z_list(1));
        z_upper_limit = in_variables_statistics(ID_INVAR_LIMITS)((z_list(1)))(2);
        z_lower_limit = in_variables_statistics(ID_INVAR_LIMITS)((z_list(1)))(1);
    end
    if (z_list(2) > 0)
        zdata = model_output(z_list(2));
        zzdata = reduce_valid_runs(zdata, valid_runs); 
        z_name = out_variables(ID_OUTVAR_NAME)(z_list(2));
        z_unit = out_variables(ID_OUTVAR_UNIT)(z_list(2));  
        z_upper_limit = out_variables(ID_OUTVAR_LIMITS)((z_list(2)))(2);
        z_lower_limit = out_variables(ID_OUTVAR_LIMITS)((z_list(2)))(1);
    end

    ui_controls(ID_AR_PLOT_HISTDIV)

    if (ui_controls(ID_PM_PLOT).Value == 2) // Histogram and variables selected 
        xlab = x_name + ", value" + " [" + x_unit + "]";
        ylab=x_name + ", class count";
        tit = "Histogram: "+x_name; 

        if (ui_controls(ID_PM_PLOT_HISTDIV).Value ~= 1)
            n_classes = ui_controls(ID_AR_PLOT_HISTDIV)(ui_controls(ID_PM_PLOT_HISTDIV).Value-1);
        else
            n_classes = round (sqrt(N_RUNS));
            if (n_classes < 5) n_classes =10; end;
        end
        try
            montesci_histplot(n_classes, xdata, xlab, ylab, tit, x_lower_limit, x_upper_limit, ui_controls(ID_PM_PLOT_FIT).value, plot_nominal, valid_runs);
        catch
            a=gca();
            close(a.parent);
            messagebox("<HTML> <BODY> Histplot failed, check the following points: <BR> Is a variable selected? <BR> Is the variable a vector? <BR> Is the variable constant?</BODY></HTML>", "Error", "error", ["Continue"], "modal")
        end
    end

    if (ui_controls(ID_PM_PLOT).Value == 3) //2D Plot
        global("fit_functions");
        xlab = x_name + " [" + x_unit + "]";
        ylab=  y_name + " [" + y_unit + "]";
        tit = x_name+ " vs. " + y_name;

        if (ui_controls(ID_PM_PLOT_FITFKT).value==1)
            fit_fkt=[];
        else
            fit_fkt = fit_functions(ui_controls(ID_PM_PLOT_FITFKT).value);    
        end
        try    
            montesci_plot2d(xdata, ydata, xlab, ylab, tit, fit_fkt, valid_runs);
        catch
            a=gca();
            close(a.parent);
            messagebox("<HTML> <BODY> 2D plot failed - check the following points: <BR> Are x and y variables selected? <BR> Are all variables scalar (not ...-vector)?</BODY></HTML>", "Error", "error", ["Continue"], "modal")
        end

    end

    if (ui_controls(ID_PM_PLOT).Value == 4) //3D Plot
        xlab = x_name + " [" + x_unit + "]";
        ylab=  y_name + " [" + y_unit + "]";
        zlab=  z_name + " [" + z_unit + "]";
        tit = x_name+ " vs. " + y_name + "and "+z_name;
        grid_values = [sqrt(N_RUNS) 10 20 40 100 200 400];
        try
            montesci_plot3d(xdata, ydata, zdata, xlab, ylab, zlab, tit, min(grid_values(ui_controls(ID_PM_PLOT_3D_GRID_PNTS).Value),N_plot_3d_max),valid_runs );
        catch
            messagebox("<HTML> <BODY> 3D plot failed - check the following points: <BR> Are x, y and z variables selected? <BR> Are all variables scalar (not ...-vector)?</BODY></HTML>", "Error", "error", ["Continue"], "modal")
        end
    end

    if (ui_controls(ID_PM_PLOT).Value == 5) //3D Plot
        xlab = x_name + " [" + x_unit + "]";
        ylab=  y_name + " [" + y_unit + "]";
        zlab=  z_name + " [" + z_unit + "]";
        tit = x_name+ " vs. " + y_name + "and "+z_name;
        grid_values = [sqrt(N_RUNS) 10 20 40 100 200 400];
        try
            montesci_surface(xdata, ydata, zdata, xlab, ylab, zlab, tit, min(grid_values(ui_controls(ID_PM_PLOT_3D_GRID_PNTS).Value),N_plot_3d_max),valid_runs );
        catch
            messagebox("<HTML> <BODY> Surface plot failed - check the following points: <BR> Are x, y and z variables selected? <BR> Are all variables scalar (not ...-vector)?</BODY></HTML>", "Error", "error", ["Continue"], "modal")
        end
    end


    if (ui_controls(ID_PM_PLOT).Value == 6) //y waveform plot
        xlab=  x_name + " [" + x_unit + "]";
        tit =  x_name + " waveform";
        try
            montesci_waveplot([], xdata, [], xlab, tit,valid_runs)
        catch
            a=gca();
            close(a.parent);
            messagebox("<HTML> <BODY> Y Wave plot failed - check the following points: <BR> Is a variable selected? <BR> Is the variable a vector (not scalar)?</BODY></HTML>", "Error", "error", ["Continue"], "modal")
        end    
    end

    if (ui_controls(ID_PM_PLOT).Value == 7) //xy Waveform plot
        xlab = x_name + " [" + x_unit + "]";
        ylab=  y_name + " [" + y_unit + "]";
        tit = x_name+ " vs. " + y_name +" waveform";
        try
            montesci_waveplot(xdata, ydata, xlab, ylab, tit, valid_runs)
        catch
            a=gca();
            close(a.parent);
            messagebox("<HTML> <BODY> XY Wave plot failed - check the following points: <BR> Are x and y variables selected? <BR> Are the variables vectors (not scalar) and of the same size?</BODY></HTML>", "Error", "error", ["Continue"], "modal")
        end
    end

    ui_controls(ID_B_PLOT).Enable ="on";

endfunction




